import dotenv from "dotenv";
dotenv.config();
import User from "../models/user.js";
import jwt from "jsonwebtoken";
import twilio from "twilio";

const JWT_SECRET = process.env.JWT_SECRET;
const accountSid = process.env.TWILIO_ACCOUNT_SID;
const authToken = process.env.TWILIO_AUTH_TOKEN;
const client = twilio(accountSid, authToken);

// Utility: Send OTP using Twilio
async function sendOtpFromTwilio(to, otp) {
  try {
    const message = await client.messages.create({
      body: `Your one-time password is ${otp}`,
      from: process.env.TWILIO_PHONE_NUMBER, // example: "+18563867972"
      to: to,
    });
    return message;
  } catch (err) {
    console.error("Error while sending twilio otp:", err);
    return err.message;
  }
}

//  Register or login with OTP
const registerLogin = async (req, res) => {
  try {
    const { countryCode, phoneNumber } = req.body;
    const full_number = countryCode.toString() + phoneNumber;

    const otp = Math.floor(100000 + Math.random() * 900000).toString();
    const otp_expiry = new Date(Date.now() + 10 * 60 * 1000); // 10 mins

    const twilioResponse = await sendOtpFromTwilio(full_number, otp);
    console.log("twilioResponse:", twilioResponse.sid || twilioResponse);

    let user = await User.findOne({ where: { full_number } });

    if (!user) {
      user = await User.create({
        country_code: countryCode,
        phone_number: phoneNumber,
        full_number: full_number,
        otp,
        otp_expiry,
      });
    } else {
      await User.update({ otp, otp_expiry }, { where: { full_number } });
      user = await User.findOne({ where: { full_number } });
    }

    res.status(201).json({
      message: "OTP sent successfully",
      user: { id: user.id, full_number },
    });
  } catch (err) {
    console.error("registerLogin error:", err);
    res.status(500).json({ error: err.message });
  }
};

//  Verify OTP
const verifyOtp = async (req, res) => {
  try {
    const { countryCode, phoneNumber, otp } = req.body;
    const full_number = countryCode.toString() + phoneNumber;
    if (!phoneNumber || !otp) {
      return res
        .status(400)
        .json({ error: "Phone number and OTP are required" });
    }

    const user = await User.findOne({ where: { full_number } });
    if (!user) {
      return res.status(404).json({ error: "User not found" });
    }

    if (user.otp !== otp || new Date() > new Date(user.otp_expiry)) {
      return res.status(400).json({ error: "Invalid or expired OTP" });
    }

    user.otp = null;
    user.otp_expiry = null;
    await user.save();
    const refreshToken = jwt.sign({ id: user.id }, JWT_SECRET, {
      expiresIn: process.env.REFRESH_TOKEN_EXPIRY,
    });
    const accessToken = jwt.sign({ id: user.id }, JWT_SECRET, {
      expiresIn: process.env.ACCESS_TOKEN_EXPIRY,
    });

    res.json({
      message: "OTP verified successfully",
      refreshToken,
      accessToken,
      user: {
        id: user.id,
        name: user.name,
        country_code: user?.country_code,
        phone_number: user?.phone_number,
        full_number: user?.full_number,
      },
    });
  } catch (err) {
    res.status(500).json({ error: err.message });
  }
};

// const login = async (req, res) => {
//   try {
//     const { phoneNumber } = req.body;

//     const user = await User.findOne({ where: { phone_number: } });
//     if (!user) {
//       return res.status(400).json({ error: "Invalid phone number" });
//     }

//     const token = sign({ id: user.id }, JWT_SECRET, { expiresIn: "1d" });

//     res.json({
//       token,
//       user: { id: user.id, name: user.name, phone_number },
//     });
//   } catch (err) {
//     res.status(500).json({ error: err.message });
//   }
// };

const triggerOtp = async (req, res) => {
  try {
    const { countryCode, phoneNumber } = req.body;

    const full_number = countryCode.toString() + phoneNumber;
    const exist = await User.findOne({ where: { full_number: full_number } });
    if (!exist) return res.status(400).json({ message: "Number Not exist" });
    const otp = Math.floor(100000 + Math.random() * 900000).toString();
    const otp_expiry = new Date(Date.now() + 10 * 60 * 1000); // 10 mins

    const twilioResponse = await sendOtpFromTwilio(full_number, otp);
    console.log("twilioResponse:", twilioResponse.sid || twilioResponse);
    await User.update(
      {
        otp: otp,
        otp_expiry: otp_expiry,
      },
      {
        where: {
          full_number: full_number,
        },
      }
    );
    res.status(200).json({ message: "Otp sent seccessfully" });
  } catch (err) {
    res.status(500).json({ error: err.message });
  }
};

const generateAccessToken = (user) => {
    try {
        return jwt.sign({ id: user.id }, process.env.JWT_SECRET, { expiresIn: process.env.ACCESS_TOKEN_EXPIRY });
    } catch (error) {
        return res.status(400).json({ message: 'Invalid or expired token' });
    }
};
 
const generateRefreshToken = (user) => {
    try {
        return jwt.sign({ id: user.id }, process.env.JWT_SECRET, { expiresIn: process.env.REFRESH_TOKEN_EXPIRY });
    } catch (err) {
        return res.status(400).json({ message: 'Invalid or expired refresh token' });
    }
};

const refreshAccessToken = async (req, res) => {
  const refreshToken = req.body.refreshToken;

  if (!refreshToken) {
    return res
      .status(400)
      .json({ message: "Refresh token not provided" });
  }

  try {
    const decoded = jwt.verify(refreshToken, process.env.JWT_SECRET);

    const user = await User.findByPk(decoded.id);
    if (!user)
      return res
        .status(404)
        .json({ message: "User not found" });

    const newAccessToken = generateAccessToken({
      id: decoded.id
    });

    return res.status(200).json({ accessToken: newAccessToken });
  } catch (err) {
    console.error("Refresh token error:", err.message);
    return res
      .status(400)
      .json({ message: "Invalid or expired refresh token" });
  }
};

export default {
  registerLogin,
  verifyOtp,
  // login,
  triggerOtp,
  refreshAccessToken,
  generateRefreshToken
};
