import { Router } from "express";
const router = Router();

import authController from "../controllers/auth.controller.js";
import validate from "../middleware/validate.js";
import {
  registerSchema,
  otpSchema,
  triggerOtpSchema,
  refreshTokenSchema,
} from "../validators/auth.validator.js";

/**
 * @swagger
 * /api/auth/register-login:
 *   post:
 *     tags:
 *       - Auth
 *     summary: Register or login user via phone number with OTP
 *     requestBody:
 *       required: true
 *       content:
 *         application/json:
 *           schema:
 *             type: object
 *             required:
 *               - countryCode
 *               - phoneNumber
 *             properties:
 *               countryCode:
 *                 type: string
 *                 example: "+91"
 *               phoneNumber:
 *                 type: string
 *                 example: "9876543210"
 *     responses:
 *       201:
 *         description: OTP sent successfully
 *         content:
 *           application/json:
 *             schema:
 *               type: object
 *               properties:
 *                 message:
 *                   type: string
 *                   example: OTP sent successfully
 *                 user:
 *                   type: object
 *                   properties:
 *                     id:
 *                       type: integer
 *                       example: 1
 *                     full_number:
 *                       type: string
 *                       example: "+919876543210"
 */

router.post(
  "/register-login",
  validate(registerSchema),
  authController.registerLogin
);

/**
 * @swagger
 * /api/auth/trigger-otp:
 *   post:
 *     tags:
 *       - Auth
 *     summary: Trigger an OTP for a given phone number
 *     requestBody:
 *       required: true
 *       content:
 *         application/json:
 *           schema:
 *             type: object
 *             required:
 *               - countryCode
 *               - phoneNumber
 *             properties:
 *               countryCode:
 *                 type: string
 *                 example: "+91"
 *               phoneNumber:
 *                 type: string
 *                 example: "9876543210"
 *     responses:
 *       200:
 *         description: OTP sent successfully
 *       400:
 *         description: Invalid input or user already exists
 */

router.post(
  "/trigger-otp",
  validate(triggerOtpSchema),
  authController.triggerOtp
);

/**
 * @swagger
 * /api/auth/verify-otp:
 *   post:
 *     tags:
 *       - Auth
 *     summary: Verify the OTP sent to a phone number
 *     requestBody:
 *       required: true
 *       content:
 *         application/json:
 *           schema:
 *             type: object
 *             required:
 *               - countryCode
 *               - phoneNumber
 *               - otp
 *             properties:
 *               countryCode:
 *                 type: string
 *                 example: "+91"
 *               phoneNumber:
 *                 type: string
 *                 example: "9876543210"
 *               otp:
 *                 type: string
 *                 example: "123456"
 *     responses:
 *       200:
 *         description: OTP verified and user authenticated
 *       400:
 *         description: Invalid or expired OTP
 *       404:
 *         description: User not found
 */

router.post("/verify-otp", validate(otpSchema), authController.verifyOtp);

/**
 * @swagger
 * /api/auth/refresh-token:
 *   post:
 *     tags:
 *       - Auth
 *     summary: Refresh access token
 *     requestBody:
 *       required: true
 *       content:
 *         application/json:
 *           schema:
 *             type: object
 *             required:
 *               - refreshToken
 *             properties:
 *               refreshToken:
 *                 type: string
 *                 example: eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9...
 *     responses:
 *       200:
 *         description: Access token refreshed successfully
 *         content:
 *           application/json:
 *             schema:
 *               type: object
 *               properties:
 *                 accessToken:
 *                   type: string
 *                   example: accessToken
 *       400:
 *         description: Invalid refresh token or schema validation failed
 */

router.post(
  "/refresh-token",
  validate(refreshTokenSchema),
  authController.refreshAccessToken
);

export default router;
